(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.fe = {})));
}(this, (function (exports) { 'use strict';

  /**
   * @function sameValueZeroEqual
   *
   * @description
   * are the objects passed strictly equal or both NaN
   *
   * @param {*} objectA the object to compare against
   * @param {*} objectB the object to test
   * @returns {boolean} are the objects equal by the SameValueZero principle
   */
  var sameValueZeroEqual = function sameValueZeroEqual(objectA, objectB) {
    return objectA === objectB || objectA !== objectA && objectB !== objectB;
  };

  /**
   * @function isPromiseLike
   *
   * @description
   * is the object promise-like (thenable)
   *
   * @param {any} object the object to test
   * @returns {boolean} is the object promise-like
   */
  var isPromiseLike = function isPromiseLike(object) {
    return typeof object.then === 'function';
  };

  /**
   * @function isReactElement
   *
   * @description
   * is the object passed a react element
   *
   * @param {any} object the object to test
   * @returns {boolean} is the object a react element
   */
  var isReactElement = function isReactElement(object) {
    return !!(object.$$typeof && object._store);
  };

  /**
   * @function toPairs
   *
   * @param {Map|Set} iterable the iterable to convert to [key, value] pairs (entries)
   * @returns {{keys: Array<*>, values: Array<*>}} the [key, value] pairs
   */
  var toPairs = function toPairs(iterable) {
    var pairs = { keys: new Array(iterable.size), values: new Array(iterable.size) };

    var index = 0;

    iterable.forEach(function (value, key) {
      pairs.keys[index] = key;
      pairs.values[index++] = value;
    });

    return pairs;
  };

  /**
   * @function areIterablesEqual
   *
   * @description
   * determine if the iterables are equivalent in value
   *
   * @param {Map|Set} objectA the object to test
   * @param {Map|Set} objectB the object to test against
   * @param {function} comparator the comparator to determine deep equality
   * @param {boolean} shouldCompareKeys should the keys be tested in the equality comparison
   * @returns {boolean} are the objects equal in value
   */
  var areIterablesEqual = function areIterablesEqual(objectA, objectB, comparator, shouldCompareKeys) {
    if (objectA.size !== objectB.size) {
      return false;
    }

    var pairsA = toPairs(objectA);
    var pairsB = toPairs(objectB);

    return shouldCompareKeys ? comparator(pairsA.keys, pairsB.keys) && comparator(pairsA.values, pairsB.values) : comparator(pairsA.values, pairsB.values);
  };

  // utils

  var HAS_MAP_SUPPORT = typeof Map === 'function';
  var HAS_SET_SUPPORT = typeof Set === 'function';

  var createComparator = function createComparator(createIsEqual) {
    var isEqual = typeof createIsEqual === 'function' ? createIsEqual(comparator) : comparator; // eslint-disable-line

    /**
     * @function comparator
     *
     * @description
     * compare the value of the two objects and return true if they are equivalent in values
     *
     * @param {*} objectA the object to test against
     * @param {*} objectB the object to test
     * @returns {boolean} are objectA and objectB equivalent in value
     */
    function comparator(objectA, objectB) {
      if (sameValueZeroEqual(objectA, objectB)) {
        return true;
      }

      var typeOfA = typeof objectA;

      if (typeOfA !== typeof objectB) {
        return false;
      }

      if (typeOfA === 'object' && objectA && objectB) {
        var arrayA = Array.isArray(objectA);
        var arrayB = Array.isArray(objectB);

        var index = void 0;

        if (arrayA || arrayB) {
          if (arrayA !== arrayB || objectA.length !== objectB.length) {
            return false;
          }

          for (index = 0; index < objectA.length; index++) {
            if (!isEqual(objectA[index], objectB[index])) {
              return false;
            }
          }

          return true;
        }

        var dateA = objectA instanceof Date;
        var dateB = objectB instanceof Date;

        if (dateA || dateB) {
          return dateA === dateB && sameValueZeroEqual(objectA.getTime(), objectB.getTime());
        }

        var regexpA = objectA instanceof RegExp;
        var regexpB = objectB instanceof RegExp;

        if (regexpA || regexpB) {
          return regexpA === regexpB && objectA.source === objectB.source && objectA.global === objectB.global && objectA.ignoreCase === objectB.ignoreCase && objectA.multiline === objectB.multiline;
        }

        if (isPromiseLike(objectA) || isPromiseLike(objectB)) {
          return objectA === objectB;
        }

        if (HAS_MAP_SUPPORT) {
          var mapA = objectA instanceof Map;
          var mapB = objectB instanceof Map;

          if (mapA || mapB) {
            return mapA === mapB && areIterablesEqual(objectA, objectB, comparator, true);
          }
        }

        if (HAS_SET_SUPPORT) {
          var setA = objectA instanceof Set;
          var setB = objectB instanceof Set;

          if (setA || setB) {
            return setA === setB && areIterablesEqual(objectA, objectB, comparator, false);
          }
        }

        var keysA = Object.keys(objectA);

        if (keysA.length !== Object.keys(objectB).length) {
          return false;
        }

        var key = void 0;

        for (index = 0; index < keysA.length; index++) {
          key = keysA[index];

          if (!Object.prototype.hasOwnProperty.call(objectB, key)) {
            return false;
          }

          // if a react element, ignore the "_owner" key because its not necessary for equality comparisons
          if (key === '_owner' && isReactElement(objectA) && isReactElement(objectB)) {
            continue;
          }

          if (!isEqual(objectA[key], objectB[key])) {
            return false;
          }
        }

        return true;
      }

      return false;
    }

    return comparator;
  };

  // comparator

  var deepEqual = createComparator();
  var shallowEqual = createComparator(function () {
    return sameValueZeroEqual;
  });

  var index = {
    createCustom: createComparator,
    deep: deepEqual,
    sameValueZero: sameValueZeroEqual,
    shallow: shallowEqual
  };

  exports.createCustomEqual = createComparator;
  exports.sameValueZeroEqual = sameValueZeroEqual;
  exports.deepEqual = deepEqual;
  exports.shallowEqual = shallowEqual;
  exports.default = index;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=fast-equals.js.map
